import * as _tiptap_core from '@tiptap/core';
import { CodeBlockOptions } from '@tiptap/extension-code-block';

interface CodeBlockLowlightOptions extends CodeBlockOptions {
    /**
     * The lowlight instance.
     */
    lowlight: any;
}
/**
 * This extension allows you to highlight code blocks with lowlight.
 * @see https://tiptap.dev/api/nodes/code-block-lowlight
 */
declare const CodeBlockLowlight: _tiptap_core.Node<CodeBlockLowlightOptions, any>;

export { CodeBlockLowlight, type CodeBlockLowlightOptions, CodeBlockLowlight as default };
